#!/usr/bin/env node

import { input, select } from '@inquirer/prompts';
import { execSync } from 'child_process';
import pkg from 'fs-extra';
import { resolve } from 'path';
import { getCommand, isPackageManager } from './utils.js';
import {
  BASIC_INTEGRATION as BASIC_INTEGRATION_MAIN,
  CONNECTOR_INTEGRATION as CONNECTOR_MAIN,
  GITIGNORE,
  createBasicIntegrationUI,
} from './constants.js';

const { ensureDirSync, writeFileSync, existsSync } = pkg;

const type = await select<string>({
  message: 'What type of integration do you want to create?',
  choices: ['Connector', 'Integration'].map((choice) => ({
    title: choice,
    value: choice.toLowerCase(),
  })),
});

const directory = await input({
  message: `Where do you want to create the ${type}?`,
  default: '.',
});

const destination = resolve(process.cwd(), directory);

const packageName = await input({
  message: `What is the slug of the ${type}? (e.g. my-integration)`,
  default: destination.split('/').pop(),
  validate: (input) => {
    // should only be lowercase letters, numbers, and dashes
    if (!/^[a-z0-9-]+$/.test(input)) {
      return 'Package name can only contain lowercase letters, numbers, and dashes';
    }
    return true;
  },
});

const packageManager = await select<string>({
  message: 'Which package manager do you want to use?',
  choices: ['npm', 'yarn', 'pnpm'].map((choice) => ({
    title: choice,
    value: choice,
  })),
});

if (!isPackageManager(packageManager)) {
  throw new Error('Invalid package manager');
}

const integrationPackageJson: Record<string, string | Record<string, string>> =
  {
    name: packageName,
    version: '0.0.1',
    main: 'src/index.ts',
    type: 'module',
    scripts: {
      build: 'ntli build -a',
      dev: 'ntli dev -a',
      preview: 'ntli preview',
    },
    dependencies: {},
  };

if (type === 'connector') {
  integrationPackageJson.scripts = {
    build: 'ntli build -c',
    dev: 'ntli dev -c',
  };
}

const integrationTsConfig = {
  compilerOptions: {
    target:
      'ES2022' /* Set the JavaScript language version for emitted JavaScript and include compatible library declarations. */,
    module: 'ES2022' /* Specify what module code is generated. */,
    moduleResolution: 'bundler',
    rootDir: '.',
  },
  exclude: ['node_modules', 'dist'],
};

ensureDirSync(destination);

if (existsSync(resolve(destination, 'package.json'))) {
  throw new Error(
    'There is seemingly already a package.json in this directory. Please remove it and try again.',
  );
}

writeFileSync(
  resolve(destination, 'package.json'),
  JSON.stringify(integrationPackageJson, null, 2),
);
writeFileSync(
  resolve(destination, 'tsconfig.json'),
  JSON.stringify(integrationTsConfig, null, 2),
);
writeFileSync(resolve(destination, '.gitignore'), GITIGNORE.trim() + '\n');

console.log('Adding dependencies...');
execSync(getCommand('add', packageManager, ['@netlify/sdk@latest']), {
  cwd: destination,
});
execSync(getCommand('add', packageManager, ['typescript@latest']), {
  cwd: destination,
});

console.log('Installing dependencies...');
execSync(getCommand('install', packageManager, []), {
  cwd: destination,
});

console.log('Initialising integration...');

execSync(
  getCommand('run', packageManager, ['ntli', 'init', '-s', packageName]),
  {
    cwd: destination,
  },
);

const exampleMains: Record<string, string> = {
  integration: BASIC_INTEGRATION_MAIN,
  connector: CONNECTOR_MAIN,
};

ensureDirSync(resolve(destination, 'src'));
writeFileSync(
  resolve(destination, 'src/index.ts'),
  '// Documentation: https://beta--developers.netlify.app\n' +
    exampleMains[type],
);

if (type === 'integration') {
  ensureDirSync(resolve(destination, 'src/ui'));
  writeFileSync(
    resolve(destination, 'src/ui/index.ts'),
    createBasicIntegrationUI(packageName),
  );
}

console.log('Done! ✅');
